@@ -2,12 +2,13 @@ package Lemonldap::NG::Common::Logger::Std;
 
 use strict;
 
-our $VERSION = '2.0.5';
+our $VERSION = '2.0.15';
 
 sub new {
     no warnings 'redefine';
     my $level = $_[1]->{logLevel} || 'info';
-    my $show = 1;
+    my $show  = 1;
+
     foreach (qw(error warn notice info debug)) {
         if ($show) {
             eval
@@ -18,7 +19,8 @@ qq'sub $_ {print STDERR "[".localtime."] [LLNG:\$\$] [$_] \$_[1]\n"}';
         }
         $show = 0 if ( $level eq $_ );
     }
-    die "unknown level $level" if ($show);
+    die "Unknown logLevel $level" if $show;
+
     return bless {}, shift;
 }
 
