@@ -2,11 +2,12 @@ package Lemonldap::NG::Common::PSGI;
 
 use strict;
 use Mouse;
+use HTTP::Status;
 use JSON;
 use Lemonldap::NG::Common::PSGI::Constants;
 use Lemonldap::NG::Common::PSGI::Request;
 
-our $VERSION = '2.0.15';
+our $VERSION = '2.22.0';
 
 our $_json = JSON->new->allow_nonref;
 
@@ -15,7 +16,6 @@ our $_json = JSON->new->allow_nonref;
 has error           => ( is => 'rw', default => '' );
 has languages       => ( is => 'rw', isa     => 'Str', default => 'en' );
 has logLevel        => ( is => 'rw', isa     => 'Str', default => 'info' );
-has portal          => ( is => 'rw', isa     => 'Str' );
 has staticPrefix    => ( is => 'rw', isa     => 'Str' );
 has instanceName    => ( is => 'rw', isa     => 'Str', default => '' );
 has customCSS       => ( is => 'rw', isa     => 'Str', default => '' );
@@ -25,6 +25,11 @@ has links           => ( is => 'rw', isa     => 'ArrayRef' );
 has menuLinks       => ( is => 'rw', isa     => 'ArrayRef' );
 has logger          => ( is => 'rw' );
 has userLogger      => ( is => 'rw' );
+has _auditLogger    => ( is => 'rw' );
+
+# portal only stores the "default" portal URL, in case a request
+# is not available in the current context to compute the dynamic portal URL
+has portal => ( is => 'rw', isa => 'Str' );
 
 # INITIALIZATION
 
@@ -37,42 +42,111 @@ sub init {
     foreach my $k ( keys %$args ) {
         $self->{$k} = $args->{$k} unless ( $k eq 'logger' );
     }
-    unless ( ref( $self->logger ) and ref( $self->userLogger ) ) {
-        my $logger =
-             $args->{logger}
-          || $ENV{LLNG_DEFAULTLOGGER}
-          || 'Lemonldap::NG::Common::Logger::Std';
-        unless ( ref $self->logger ) {
-            eval "require $logger";
-            die $@ if ($@);
-            my $err;
-            unless ( $self->{logLevel} =~ /^(?:debug|info|notice|warn|error)$/ )
-            {
-                $err =
-                    'Bad logLevel value \''
-                  . $self->{logLevel}
-                  . "', switching to 'info'";
-                $self->{logLevel} = 'info';
-            }
-            $self->logger( $logger->new($self) );
-            $self->logger->error($err) if $err;
+    $self->_initLogger($args);
+
+    return 1;
+}
+
+sub _initLogger {
+    my ( $self, $args ) = @_;
+
+    my $logger =
+         $args->{logger}
+      || $ENV{LLNG_DEFAULTLOGGER}
+      || 'Lemonldap::NG::Common::Logger::Std';
+    unless ( ref $self->logger ) {
+        eval "require $logger";
+        die $@ if ($@);
+        my $err;
+        unless ( $self->{logLevel} =~ /^(?:debug|info|notice|warn|error)$/ ) {
+            $err =
+                'Bad logLevel value \''
+              . $self->{logLevel}
+              . "', switching to 'info'";
+            $self->{logLevel} = 'info';
         }
-        unless ( ref $self->userLogger ) {
-            $logger = $ENV{LLNG_USERLOGGER} || $args->{userLogger} || $logger;
-            eval "require $logger";
-            die $@ if ($@);
-            require Lemonldap::NG::Common::Logger::_Duplicate;
-            $self->userLogger(
-                Lemonldap::NG::Common::Logger::_Duplicate->new(
-                    $self,
-                    user   => 1,
-                    logger => $logger,
-                    dup    => $self->logger
-                )
-            );
+        $self->logger( $logger->new($self) );
+        $self->logger->error($err) if $err;
+    }
+
+    unless ( ref $self->userLogger ) {
+        $logger = $ENV{LLNG_USERLOGGER} || $args->{userLogger} || $logger;
+        eval "require $logger";
+        die $@ if ($@);
+        require Lemonldap::NG::Common::Logger::_Duplicate;
+        $self->userLogger(
+            Lemonldap::NG::Common::Logger::_Duplicate->new(
+                $self,
+                user   => 1,
+                logger => $logger,
+                dup    => $self->logger
+            )
+        );
+    }
+
+    unless ( ref $self->_auditLogger ) {
+        $logger =
+             $ENV{LLNG_AUDITLOGGER}
+          || $args->{auditLogger}
+          || "Lemonldap::NG::Common::AuditLogger::UserLoggerCompat";
+        eval "require $logger";
+        die $@ if ($@);
+        $self->_auditLogger( $logger->new($self) );
+    }
+}
+
+sub getLanguage {
+    my ( $self, $req ) = @_;
+
+    # Get the list of allowed languages from config
+    my @langArray = split( qr/[ ,]+/, $self->languages );
+
+    # If the language in the cookie is found in allowed languages, use it
+    my $cookie_lang = $req->cookies->{llnglanguage};
+    if ( $cookie_lang and grep { $_ eq $cookie_lang } @langArray ) {
+        return $cookie_lang;
+    }
+
+    # Get ordered list of preferred languages from Accept-Language header
+    my @preferred_req_language = $self->_getLanguageListFromHeader($req);
+
+    # Go through preferred languages and pick the first allowed one
+    for my $req_lang (@preferred_req_language) {
+        if ( grep { lc($_) eq lc($req_lang) } @langArray ) {
+            return $req_lang;
         }
     }
-    return 1;
+
+    # fallback: pick the first allowed language, default to english
+    return ( $langArray[0] || 'en' );
+}
+
+sub _getLanguageListFromHeader {
+    my ( $self, $req ) = @_;
+
+    # Extract [2 letter code, priority] from list of Accept-Language values
+    my @req_languages_and_prio = map {
+        my ( $lang, $prio ) = m/^(\w+)(?:.*q=(\d\.[\d+]))?/;
+        $prio ||= 1;
+
+        # Skip invalid entries
+        $lang ? ( [ $lang, $prio ] ) : ()
+      }
+      split( qr/\s*,\s*/, $req->languages || "" );
+
+    # Sort by priority (perl sort is stable by default)
+    @req_languages_and_prio =
+      sort { $b->[1] <=> $a->[1] } @req_languages_and_prio;
+
+    # Keep only language code
+    my @preferred_req_language = map { $_->[0] } @req_languages_and_prio;
+
+    return @preferred_req_language;
+}
+
+sub auditLog {
+    my ( $self, $req, %info ) = @_;
+    $self->_auditLogger->log( $req, %info );
 }
 
 # RUNNING METHODS
@@ -156,13 +230,21 @@ sub sendJSONresponse {
         };
         return $self->sendError( $req, $@ ) if ($@);
     }
-    return [ $args{code}, [ 'Content-Type' => $type, @{ $args{headers} } ],
-        [$j] ];
+    utf8::downgrade($j);
+    return [
+        $args{code},
+        [
+            'Content-Type'   => $type,
+            'Content-Length' => length($j),
+            @{ $args{headers} }
+        ],
+        [$j]
+    ];
 }
 
 sub sendError {
     my ( $self, $req, $err, $code ) = @_;
-    $err  ||= $req->error;
+    $err  ||= $req->error || '';
     $code ||= 500;
     $self->lmLog( "Error $code: $err", $code > 499 ? 'error' : 'notice' );
 
@@ -189,20 +271,15 @@ sub sendError {
     }
 
     # Handle Ajax responses
-    elsif ( $req->accept =~ /json/ ) {
+    elsif ( ( $req->accept // "" ) =~ /json/ ) {
         return $self->sendJSONresponse( $req, { error => $err },
             code => $code );
     }
 
     # Default response: HTML
     else {
-        my $title = (
-              $code >= 500 ? 'Server error'
-            : $code == 403 ? 'Forbidden'
-            : $code == 401 ? 'Authentication required'
-            : $code == 400 ? 'Bad request'
-            :                'Error'
-        );
+        my $title = eval { HTTP::Status::status_message($code) } || 'Error';
+        print STDERR $@ if $@;
 
         # TODO: this should probably use a template instead
         my $s = "<html><head><title>$title</title>
@@ -238,10 +315,11 @@ sub sendRawHtml {
 sub abort {
     my ( $self, $err ) = @_;
     eval { $self->logger->error($err) };
-    return sub {
-        $self->sendError( Lemonldap::NG::Common::PSGI::Request->new( $_[0] ),
-            $err, 500 );
-    };
+    return $self->psgiAdapter(
+        sub {
+            $self->sendError( $_[0], $err, 500 );
+        }
+    );
 }
 
 sub _mustBeDefined {
@@ -260,13 +338,16 @@ sub sendJs {
     $sp =~ s/\/*$/\//;
     my $sc = $req->script_name // "";
 
+    my $portal = $req->can('portal') ? $req->portal : $self->portal;
+
     # Javascript scriptname is assumed by our JS code to end with /
     $sc =~ s#/*$#/#;
     my $s =
         sprintf 'var staticPrefix="%s";'
       . 'var scriptname="%s";'
       . 'var availableLanguages="%s".split(/[,;] */);'
-      . 'var portal="%s";', $sp, $sc, $self->languages, $self->portal;
+      . 'var portal="%s";', $sp, $sc, $self->languages,
+      $portal;
     $s .= $self->javascript($req) if ( $self->can('javascript') );
     return [
         200,
@@ -290,15 +371,12 @@ sub sendHtml {
     $args{code}    ||= 200;
     $args{headers} ||= [ $req->spliceHdrs ];
     my $htpl;
-    $template = ( $args{templateDir} // $self->templateDir ) . "/$template.tpl";
-    return $self->sendError( $req, "Unable to read $template", 500 )
-      unless ( -r $template and -f $template );
     eval {
         $self->logger->debug("Starting HTML generation using $template");
         require HTML::Template;
         $htpl = HTML::Template->new(
-            filehandle             => IO::File->new($template),
-            path                   => $self->templateDir,
+            filename => $template . ".tpl",
+            path     => ( $args{templateDir} // $self->templateDir ),
             search_path_on_include => 1,
             die_on_bad_params      => 0,
             die_on_missing_include => 1,
@@ -323,7 +401,9 @@ sub sendHtml {
         );
     };
     if ($@) {
-        return $self->sendError( $req, "Unable to load template: $@", 500 );
+        $self->logger->error("Unable to render template $template.tpl: $@");
+        return $self->sendError( $req,
+            "Unable to render template $template.tpl", 500 );
     }
 
     # Set headers
@@ -341,21 +421,34 @@ sub run {
     $args //= {};
     unless ( ref $self ) {
         $self = $self->new($args);
-        return $self->abort( $self->error ) unless ( $self->init($args) );
+        $self->init($args) or $self->logger->error('Initialization failed');
     }
     return $self->_run;
 }
 
 sub _run {
     my $self = shift;
-    return sub {
-        $self->_logAndHandle(
-            Lemonldap::NG::Common::PSGI::Request->new( $_[0] ) );
-    };
+    return $self->psgiAdapter(
+        sub {
+            $self->handler( $_[0] );
+        }
+    );
 }
 
-sub _logAndHandle {
-    my ( $self, $req ) = @_;
+# This method returns a sub that takes a Lemonldap::NG::Common::PSGI::Request
+# object and returns a PSGI response into a proper PSGI method.
+sub psgiAdapter {
+    my ( $self, $sub ) = @_;
+    return sub {
+        my $env = shift;
+        my $req = Lemonldap::NG::Common::PSGI::Request->new($env);
+        return $self->logAndRun( $req, $sub );
+    }
+}
+
+# This method sets up LemonLDAP::NG logging for the current request
+sub logAndRun {
+    my ( $self, $req, $sub ) = @_;
 
     # register the request object to the logging system
     if ( ref( $self->logger ) and $self->logger->can('setRequestObj') ) {
@@ -366,8 +459,13 @@ sub _logAndHandle {
         $self->userLogger->setRequestObj($req);
     }
 
-    # Call the handler
-    my $res = $self->handler($req);
+    if ( ref $self->logger ) {
+        $self->logger->info( "New request "
+              . ref($self) . " "
+              . $req->method . " "
+              . $req->request_uri );
+    }
+    my $res = $sub->($req);
 
     # Clear the logging system before the next request
     if ( ref( $self->logger ) and $self->logger->can('clearRequestObj') ) {
@@ -406,7 +504,7 @@ Use Lemonldap::NG::Common::PSGI::Router for REST API.
     $self->logLevel('info');
     # It is possible to use syslog for user actions
     $self->syslog('daemon');
-  
+
     # Return a boolean. If false, then error message has to be stored in
     # $self->error
     return 1;
@@ -416,7 +514,7 @@ Use Lemonldap::NG::Common::PSGI::Router for REST API.
     my ( $self, $req ) = @_;
     # Do something and return a PSGI response
     # NB: $req is a Lemonldap::NG::Common::PSGI::Request object
-    
+  
     return [ 200, [ 'Content-Type' => 'text/plain' ], [ 'Body lines' ] ];
   }
 
@@ -426,7 +524,7 @@ This package could then be called as a CGI, using FastCGI,...
   
   use My::PSGI;
   use Plack::Handler::FCGI; # or Plack::Handler::CGI
-
+  
   Plack::Handler::FCGI->new->run( My::PSGI->run() );
 
 =head1 DESCRIPTION
