@@ -23,11 +23,13 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_UNAUTHORIZEDPARTNER
 );
 
-our $VERSION = '2.21.0';
+our $VERSION = '2.22.0';
 
 extends 'Lemonldap::NG::Portal::Main::Issuer',
   'Lemonldap::NG::Portal::Lib::SAML';
 
+with 'Lemonldap::NG::Portal::Lib::Key';
+
 has rule           => ( is => 'rw' );
 has ssoUrlRe       => ( is => 'rw' );
 has ssoUrlArtifact => ( is => 'rw' );
@@ -197,7 +199,8 @@ sub storeEnv {
             $req->env->{llng_saml_spconfkey} = $spConfKey;
 
             # Store target authentication level in pdata
-            my $targetAuthnLevel = $self->spLevelRules->{$spConfKey}->( $req, {} );
+            my $targetAuthnLevel =
+              $self->spLevelRules->{$spConfKey}->( $req, {} );
 
             $req->pdata->{targetAuthnLevel} = $targetAuthnLevel
               if $targetAuthnLevel;
@@ -563,7 +566,8 @@ sub run {
             if ( $login->request()->NameIDPolicy ) {
                 $nameIDFormat = $login->request()->NameIDPolicy->Format();
                 $self->logger->debug(
-                    "Get NameID format $nameIDFormat from request") if defined($nameIDFormat);
+                    "Get NameID format $nameIDFormat from request")
+                  if defined($nameIDFormat);
             }
 
             # Get default NameID Format from configuration
@@ -639,7 +643,7 @@ sub run {
             if (
                 $force_authn
                 and (
-                    time - $req->sessionInfo->{_utime} >
+                    time - $req->sessionInfo->{_lastAuthnUTime} >
                     $self->conf->{portalForceAuthnInterval} )
               )
             {
@@ -2026,7 +2030,7 @@ sub sloServer {
 
         if ($session_index) {
             my $sessionIndexSession = $self->getSamlSession($session_index);
-            return $self->p->do( $req, [ sub { PE_SESSIONEXPIRED } ] )
+            return $self->p->doPE( $req, PE_SESSIONEXPIRED )
               unless $sessionIndexSession;
 
             $local_session_id = $sessionIndexSession->data->{_saml_id};
@@ -2149,17 +2153,17 @@ sub sloServer {
         my $doAuthLogout = 0;
         my $logoutContextSession;
 
-        # TODO: for now, we only try authLogout to disconnect an
-        # external IDP if the current session has been opened on a
-        # SAML IDP. We only propagate logout to the IDP our SP is SAML
-        # AND our IDP is SAML too
-        if ( $req->sessionInfo->{_lassoSessionDump} ) {
+        # TODO: only propagation to SAML or OIDC issuers is supported
+        # at the moment
+        if (   $req->sessionInfo->{_lassoSessionDump}
+            or $req->sessionInfo->{_oidc_OP} )
+        {
 
             $doAuthLogout = 1;
 
             # In case we have to redirect to the IDP, save current state
             # to allow resumption. This needs to be done here because
-            # issuerUrldc will be put in the IdP logout's RelayState
+            # issuerNextUrl will be put in the IdP logout's RelayState
             my $logoutInfos = {
                 logout      => $logout->dump,
                 spConfKey   => $spConfKey,
@@ -2176,7 +2180,7 @@ sub sloServer {
                 )
             );
             $uri->query_param( ResumeParams => $logoutContextSession->id );
-            $req->{issuerUrldc} = $uri->as_string;
+            $req->{issuerNextUrl} = $uri->as_string;
         }
 
         # We don't want info to interfere with the auth logout process
@@ -2201,7 +2205,7 @@ sub sloServer {
             if ($savedInfo) {
                 $logoutContextSession->update( { info => $savedInfo } );
             }
-            return $self->p->do( $req, [ sub { PE_REDIRECT } ] );
+            return $self->p->doPE( $req, PE_REDIRECT );
         }
 
         $req->info($savedInfo);
