@@ -8,7 +8,7 @@
 #                  of lemonldap-ng.ini) and underlying handler configuration
 package Lemonldap::NG::Portal::Main::Init;
 
-our $VERSION = '2.17.2';
+our $VERSION = '2.22.0';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -16,21 +16,33 @@ use strict;
 use Mouse;
 use Regexp::Assemble;
 use Lemonldap::NG::Common::Util qw(getSameSite);
+use URI;
+use Lemonldap::NG::Portal;
+use MIME::Base64;
+use Digest::SHA;
 
 # PROPERTIES
 
 # Configuration storage
 has localConfig => ( is => 'rw', default => sub { {} } );
 has conf        => ( is => 'rw', default => sub { {} } );
-has menu        => ( is => 'rw', default => sub { {} } );
 has trOver      => ( is => 'rw', default => sub { { all => {} } } );
 
 # Sub modules
 has _authentication => ( is => 'rw' );
 has _userDB         => ( is => 'rw' );
 has _passwordDB     => ( is => 'rw' );
-has _sfEngine       => ( is => 'rw' );
-has _captcha        => ( is => 'rw' );
+
+has _loadedServices => ( is => 'rw', default => sub { {} } );
+has _loadedPlugins  => ( is => 'rw', default => sub { {} } );
+
+# Legacy
+sub _captcha        { $_[0]->getService('captcha') }
+sub _trustedBrowser { $_[0]->getService('trustedBrowser') }
+sub _sfEngine       { $_[0]->getService('secondFactor') }
+sub menu            { $_[0]->getService('menu') }
+
+has _ppRules => ( is => 'rw', default => sub { {} } );
 
 has loadedModules => ( is => 'rw' );
 
@@ -43,6 +55,12 @@ has _jsRedirect => ( is => 'rw' );
 has trustedDomainsRe         => ( is => 'rw' );
 has additionalTrustedDomains => ( is => 'rw', default => sub { [] } );
 
+has cacheTag => ( is => 'rw' );
+
+# Entrypoints
+has _pluginEntryPoints =>
+  ( is => 'rw', isa => 'ArrayRef', default => sub { [] } );
+
 # Lists to store plugins entry-points
 my @entryPoints;
 
@@ -72,11 +90,11 @@ BEGIN {
 }
 
 # Endpoints inserted after any main sub
-has 'afterSub'  => ( is => 'rw', default => sub { {} } );
-has 'aroundSub' => ( is => 'rw', default => sub { {} } );
+has afterSub  => ( is => 'rw', default => sub { {} } );
+has aroundSub => ( is => 'rw', default => sub { {} } );
 
 # Issuer hooks
-has 'hook' => ( is => 'rw', default => sub { {} } );
+has hook => ( is => 'rw', default => sub { {} } );
 
 has spRules => (
     is      => 'rw',
@@ -101,6 +119,24 @@ has pluginSessionDataToRemember =>
 
 # INITIALIZATION
 
+sub _resetPluginsAndServices {
+    my ($self) = @_;
+    $self->loadedModules( {} );
+    $self->_loadedServices( {} );
+    $self->_loadedPlugins( {} );
+    $self->afterSub( {} );
+    $self->aroundSub( {} );
+    $self->spRules( {} );
+    $self->hook( {} );
+    $self->pluginSessionDataToRemember( {} );
+    $self->_pluginEntryPoints( [] );
+
+    # Reinitialize arrays
+    foreach ( qw(_macros _groups), @entryPoints ) {
+        $self->{$_} = [];
+    }
+}
+
 sub init {
     my ( $self, $args ) = @_;
     $args ||= {};
@@ -131,20 +167,23 @@ sub init {
     $self->trOver( JSON::to_json( $self->trOver ) );
 
     # Purge loaded module list
-    $self->loadedModules( {} );
-    $self->afterSub( {} );
-    $self->aroundSub( {} );
-    $self->hook( {} );
+    $self->_resetPluginsAndServices;
 
     # Insert `reloadConf` in handler reload stack
     Lemonldap::NG::Handler::Main->onReload( $self, 'reloadConf' );
 
+    # Register logout event (unlog event is only a local unlog: clean cache)
+    &Lemonldap::NG::Handler::Main::MsgActions::addMsgAction(
+        'logout',
+        sub {
+            my ( $class, $id, $req ) = @_;
+            return $self->eventLogout( $req, $id );
+        }
+    );
+
     # Handler::PSGI::Try initialization
     unless ( $self->SUPER::init( $self->localConfig ) ) {
         $self->logger->error( 'Initialization failed: ' . $self->error );
-        $self->error(
-"Initialization failed! Enable debug logs, reload your web server and catch main error..."
-        );
         return 0;
     }
     if ( $self->error ) {
@@ -222,23 +261,17 @@ sub setPortalRoutes {
 sub reloadConf {
     my ( $self, $conf ) = @_;
 
+    # Store default portal value in $self->portal
+    $self->portal( Lemonldap::NG::Handler::Main->tsv->{portal}->() );
+
     # Handle requests (other path may be declared in enabled plugins)
     $self->setPortalRoutes;
 
     # Reinitialize $self->conf
     %{ $self->{conf} } = %{ $self->localConfig };
 
-    # Reinitialize arrays
-    foreach ( qw(_macros _groups), @entryPoints ) {
-        $self->{$_} = [];
-    }
-    $self->afterSub( {} );
-    $self->aroundSub( {} );
-    $self->spRules( {} );
-    $self->hook( {} );
-
-    # Plugin history fields
-    $self->pluginSessionDataToRemember( {} );
+    # Purge loaded module list
+    $self->_resetPluginsAndServices;
 
     # Load conf in portal object
     foreach my $key ( keys %$conf ) {
@@ -271,17 +304,6 @@ sub reloadConf {
     $self->cors($cors);
     $self->logger->debug( "Initialized CORS headers : " . $self->cors );
 
-    # Initialize templateDir
-    $self->{templateDir} =
-      $self->conf->{templateDir} . '/' . $self->conf->{portalSkin}
-      if ( $self->conf->{templateDir} and $self->conf->{portalSkin} );
-    unless ( -d $self->{templateDir} ) {
-        $self->error("Template dir $self->{templateDir} doesn't exist");
-        return $self->fail;
-    }
-    $self->templateDir(
-        [ $self->{templateDir}, $self->conf->{templateDir} . '/bootstrap' ] );
-
     $self->{staticPrefix} = $self->conf->{staticPrefix} || '/static';
     $self->{languages}    = $self->conf->{languages}    || 'en';
 
@@ -300,21 +322,26 @@ sub reloadConf {
           $self->conf->{globalStorageOptions};
     }
 
-    # Initialize cookie domain
-    unless ( $self->conf->{domain} ) {
-        $self->error('Configuration error: no domain');
-        return $self->fail;
-    }
-    $self->conf->{domain} =~ s/^([^\.])/.$1/;
-
     # Initialize cookie SameSite value
     $self->cookieSameSite( getSameSite( $self->conf ) );
     $self->logger->debug(
         "Cookies will use SameSite=" . $self->cookieSameSite );
 
+    # Set asset tag from version and optional salt
+    my $cacheTagSalt = $self->conf->{cacheTagSalt} // "";
+    my $key          = $self->conf->{key}          // "";
+    my $digest = substr(
+        MIME::Base64::encode_base64url(
+            Digest::SHA::hmac_sha256(
+                $Lemonldap::NG::Portal::VERSION . $cacheTagSalt, $key
+            )
+        ),
+        0, 8
+    );
+    $self->cacheTag($digest);
+
     # Load menu
     # ---------
-    $self->menu( $self->loadPlugin('::Main::Menu') );
     $self->displayInit;
 
     # Load authentication/userDB
@@ -335,19 +362,6 @@ sub reloadConf {
           unless ( $self->{"_$type"} = $self->loadPlugin($module) );
     }
 
-    # Load second-factor engine
-    return $self->fail
-      unless $self->{_sfEngine} =
-      $self->loadPlugin( $self->conf->{'sfEngine'} );
-
-    # Load Captcha module
-    return $self->fail
-      unless $self->_captcha(
-        $self->loadPlugin(
-            $self->conf->{'captcha'} || '::Captcha::SecurityImage'
-        )
-      );
-
     # Compile macros in _macros, groups in _groups
     foreach my $type (qw(macros groups)) {
         $self->{"_$type"} = {};
@@ -371,6 +385,11 @@ sub reloadConf {
       or $self->logger->error(
         'jsRedirect returns an error: ' . HANDLER->tsv->{jail}->error );
 
+    # Load services
+    foreach my $service ( $self->enabledServices ) {
+        $self->loadService(@$service) or return $self->fail;
+    }
+
     # Load plugins
     foreach my $plugin ( $self->enabledPlugins ) {
         $self->loadPlugin($plugin) or return $self->fail;
@@ -402,15 +421,14 @@ sub reloadConf {
                 $re->add("$_");
             }
         }
-        foreach ( @{ $self->{additionalTrustedDomains} },
-            $self->conf->{portal} )
-        {
+        my $default_portal = HANDLER->tsv->{portal}->();
+        foreach ( @{ $self->{additionalTrustedDomains} }, $default_portal ) {
             my $p = $_;
             $p =~ s#https?://([^/]*).*$#$1#;
             $re->add( quotemeta($p) );
         }
 
-        foreach my $vhost ( keys %{ $self->conf->{locationRules} } ) {
+        foreach my $vhost ( sort keys %{ $self->conf->{locationRules} } ) {
             my $expr = quotemeta($vhost);
 
             # Handle wildcards
@@ -438,47 +456,55 @@ sub reloadConf {
     }
 
     # Clean $req->pdata after authentication
-    push @{ $self->endAuth }, sub {
-        my $tmp = $_[0]->pdata->{keepPdata} //= [];
-        foreach my $k ( keys %{ $_[0]->pdata } ) {
-            unless ( grep { $_ eq $k } @$tmp ) {
-                $self->logger->debug("Removing $k from pdata");
-                delete $_[0]->pdata->{$k};
-            }
-        }
-        my $user_log = $_[0]->{sessionInfo}->{ $self->conf->{whatToTrace} };
-        $self->userLogger->notice( $user_log . ' connected' ) if $user_log;
-        if (@$tmp) {
-            $self->logger->debug(
-                'Add ' . join( ',', @$tmp ) . ' in keepPdata' );
-            $_[0]->pdata->{keepPdata} = $tmp;
-        }
-        return PE_OK;
-    };
-    unshift @{ $self->beforeAuth }, sub {
-        if ( $_[0]->param('cancel') ) {
-            $self->logger->debug('Cancel called, push authCancel calls');
-            unshift @{ $_[0]->steps }, @{ $self->authCancel };
-            return PE_OK;
-        }
-    };
-    my $portal = $self->conf->{portal};
-    $portal =~ s#^https?://(.*?)(?:[:/].*)?$#$1#;
-    HANDLER->tsv->{defaultCondition}->{$portal} ||= sub { 1 };
+    push @{ $self->endAuth }, "cleanPdata";
+
+    # Failsafe: allow handler access to portal in case it's missing in conf
+    my $default_portal_uri  = URI->new( HANDLER->tsv->{portal}->() );
+    my $default_portal_host = eval { $default_portal_uri->host };
+    if ($default_portal_host) {
+        HANDLER->tsv->{defaultCondition}->{$default_portal_host} ||= sub { 1 };
+    }
 
     1;
 }
 
 # Method used to load plugins
+
+sub loadService {
+    my ( $self, $name, $plugin ) = @_;
+    $self->logger->debug("Loading service $name from $plugin");
+    return $self->_loadedServices->{$name} = $self->loadPlugin($plugin);
+}
+
+sub getService {
+    my ( $self, $name ) = @_;
+    return $_[0]->_loadedServices->{$name};
+}
+
 sub loadPlugin {
     my ( $self, $plugin ) = @_;
     unless ($plugin) {
         require Carp;
-        Carp::confess('Calling loadPugin without arg !');
+        Carp::confess('Calling loadPugin without arg!');
     }
-    my $obj;
-    return 0
-      unless ( $obj = $self->loadModule("$plugin") );
+
+    my $full_name =
+      ( $plugin =~ /^::/ ) ? "Lemonldap::NG::Portal$plugin" : $plugin;
+    if (
+        grep {
+            my $item_full_name =
+              ( $_ =~ /^::/ ) ? "Lemonldap::NG::Portal$_" : $_;
+            $item_full_name eq $full_name
+        } split( /[\s,]+/, $self->conf->{disabledPlugins} // "" )
+      )
+    {
+        $self->logger->debug(
+            "Module $plugin is disallowed by disabledPlugins and was not loaded"
+        );
+        return 1;
+    }
+
+    return 0 unless ( my $obj = $self->loadModule($plugin) );
     return $self->findEP( $plugin, $obj );
 }
 
@@ -486,23 +512,45 @@ sub loadPlugin {
 sub findEP {
     my ( $self, $plugin, $obj ) = @_;
 
+    my $module = $plugin =~ /^::/ ? "Lemonldap::NG::Portal$plugin" : $plugin;
+
+    # Register the plugin's instance so that callbacks can be refered by a
+    # serializable string
+    # DEPRECATED: is_only=0 should no longer be supported after LLNG 3.
+    # Corresponding code paths must be removed
+    my $is_only = 1;
+    if ( $self->_loadedPlugins->{$module} ) {
+        $self->logger->debug( "Plugin $module has already been loaded once." );
+        $is_only = 0;
+    }
+    else {
+        $self->_loadedPlugins->{$module} = $obj;
+    }
+
     # Standards entry points
     foreach my $sub (@entryPoints) {
         if ( $obj->can($sub) ) {
-            $self->logger->debug(" Found $sub entry point:");
+            $self->logger->debug(" Found $sub entry point in $module");
             if ( my $callback = $obj->$sub ) {
-                push @{ $self->{$sub} }, sub {
-                    eval {
-                        $obj->logger->debug("Launching ${plugin}::$callback");
+
+                if ($is_only) {
+                    push @{ $self->{$sub} }, "${module}::${callback}";
+                }
+                else {
+                    push @{ $self->{$sub} }, sub {
+                        eval {
+                            $obj->logger->debug(
+                                "Launching ${module}::$callback");
+                        };
+                        $obj->$callback(@_);
                     };
-                    $obj->$callback(@_);
-                };
+                }
                 $self->logger->debug("  -> $callback");
             }
         }
     }
     if ( $obj->can('afterSub') ) {
-        $self->logger->debug("Found afterSub in $plugin");
+        $self->logger->debug("Found afterSub in $module");
         my $h = $obj->afterSub;
         unless ( ref $h and ref($h) eq 'HASH' ) {
             $self->logger->error(
@@ -511,18 +559,23 @@ sub findEP {
         else {
             foreach my $ep ( keys %$h ) {
                 my $callback = $h->{$ep};
-                push @{ $self->afterSub->{$ep} }, sub {
-                    eval {
-                        $obj->logger->debug(
-                            "Launching ${plugin}::$callback afterSub $ep");
+                if ($is_only) {
+                    push @{ $self->afterSub->{$ep} }, "${module}::${callback}";
+                }
+                else {
+                    push @{ $self->afterSub->{$ep} }, sub {
+                        eval {
+                            $obj->logger->debug(
+                                "Launching ${module}::$callback afterSub $ep");
+                        };
+                        $obj->$callback(@_);
                     };
-                    $obj->$callback(@_);
-                };
+                }
             }
         }
     }
     if ( $obj->can('aroundSub') ) {
-        $self->logger->debug("Found aroundSub in $plugin");
+        $self->logger->debug("Found aroundSub in $module");
         my $h = $obj->aroundSub;
         unless ( ref $h and ref($h) eq 'HASH' ) {
             $self->logger->error(
@@ -533,19 +586,19 @@ sub findEP {
                 my $callback    = $h->{$ep};
                 my $previousSub = $self->aroundSub->{$ep} ||= sub {
                     $self->logger->debug(
-                        "$ep launched inside ${plugin}::$callback");
+                        "$ep launched inside ${module}::$callback");
                     $self->$ep(@_);
                 };
                 $self->aroundSub->{$ep} = sub {
                     $self->logger->debug(
-                        "Launching ${plugin}::$callback instead of $ep");
+                        "Launching ${module}::$callback instead of $ep");
                     $obj->$callback( $previousSub, @_ );
                 };
             }
         }
     }
     if ( $obj->can('hook') ) {
-        $self->logger->debug("Found hook in $plugin");
+        $self->logger->debug("Found hook in $module");
         my $h = $obj->hook;
         unless ( ref $h and ref($h) eq 'HASH' ) {
             $self->logger->error('"hook" endpoint must be a hashref, skipped');
@@ -556,24 +609,62 @@ sub findEP {
                 push @{ $self->hook->{$hookname} }, sub {
                     eval {
                         $obj->logger->debug(
-                            "Launching ${plugin}::$callback on hook $hookname");
+                            "Launching ${module}::$callback on hook $hookname");
                     };
                     $obj->$callback(@_);
                 };
             }
         }
     }
-    $self->logger->debug("Plugin $plugin initialized");
 
     # Rules for menu
     if ( $obj->can('spRules') ) {
-        foreach my $k ( keys %{ $obj->{spRules} } ) {
+        foreach my $k ( keys %{ $obj->spRules } ) {
             $self->logger->info(
 "$k is defined more than one time, it can have some bad effects on Menu display"
             ) if ( $self->spRules->{$k} );
-            $self->spRules->{$k} = $obj->{spRules}->{$k};
+            $self->spRules->{$k} = $obj->spRules->{$k};
         }
     }
+
+    # Plugin entrypoints
+    for my $ep ( @{ $self->_pluginEntryPoints } ) {
+        if (   ( $ep->{can} and $obj->can( $ep->{can} ) )
+            or ( $ep->{isa}  and $obj->isa( $ep->{isa} ) )
+            or ( $ep->{does} and $obj->does( $ep->{does} ) ) )
+        {
+            my @args = @{ $ep->{args} || [] };
+            if ( my $callback = $ep->{callback} ) {
+                $self->logger->debug(
+                    "Invoking callback registered by $ep->{_pkg}");
+                $callback->( $obj, @args );
+            }
+            elsif ( $ep->{service} && $ep->{method} ) {
+                my $service = $self->getService( $ep->{service} );
+                if ($service) {
+                    if ( my $method = $service->can( $ep->{method} ) ) {
+                        $self->logger->debug(
+                                "Invoking $ep->{method} on $ep->{service}"
+                              . " on behalf of $ep->{_pkg}" );
+                        $service->$method( $obj, @args );
+                    }
+                    else {
+                        $self->logger->warn(
+                            "Service $ep->{service} has no $ep->{method} method"
+                              . " in entrypoint added by $ep->{_pkg}" );
+                    }
+                }
+                else {
+                    $self->logger->warn(
+                            "Could not find service $ep->{service}"
+                          . " in entrypoint added by $ep->{_pkg}" );
+                }
+            }
+        }
+    }
+
+    $self->logger->debug("Plugin $module initialized");
+
     return $obj;
 }
 
@@ -583,10 +674,14 @@ sub loadModule {
     my $obj;
     $module = "Lemonldap::NG::Portal$module" if ( $module =~ /^::/ );
 
-    eval "require $module";
-    if ($@) {
-        $self->logger->error("$module load error: $@");
-        return 0;
+    # If module is not already in namespace, try to load it
+    unless ( $module->can('new') ) {
+        eval "require $module";
+        if ($@) {
+            $self->logger->error("$module load error: $@");
+            $self->error("$module load error: $@");
+            return 0;
+        }
     }
     eval {
         $obj = $module->new( { p => $self, conf => $conf, %args } );
@@ -602,6 +697,7 @@ sub loadModule {
     }
     if ( $obj->can("init") and ( !$obj->init ) ) {
         $self->logger->error("$module init failed");
+        $self->error("$module init failed");
         return 0;
     }
 
@@ -610,7 +706,7 @@ sub loadModule {
 }
 
 sub fail {
-    $_[0]->userLogger->error( $_[0]->error );
+    $_[0]->logger->error( $_[0]->error );
     $_[0]->addUnauthRoute( '*' => 'displayError' );
     $_[0]->addAuthRoute( '*' => 'displayError' );
     return 0;
@@ -628,20 +724,36 @@ sub displayError {
 # returns undef if the rule syntax was invalid
 sub buildRule {
     my ( $self, $rule, $ruleDesc ) = @_;
-    if ($ruleDesc) {
-        $ruleDesc = " $ruleDesc ";
-    }
-    else {
-        $ruleDesc = " ";
-    }
+    $ruleDesc ||= '';
     my $compiledRule =
       $self->HANDLER->buildSub( $self->HANDLER->substitute($rule) );
     unless ($compiledRule) {
         my $error =
           $self->HANDLER->tsv->{jail}->error || 'Unable to compile rule';
-        $self->logger->error( "Bad" . $ruleDesc . "rule: " . $error );
+        $self->logger->error("Bad $ruleDesc rule: $error");
+        return undef;
     }
-    return $compiledRule,;
+
+    # Avoid deep recursion
+    my $overLoadedRule = $compiledRule;
+    if ( $self->conf->{logParams} ) {
+        $overLoadedRule = sub {
+            $self->_dump($_[0]);
+            return $compiledRule->(@_);
+        };
+    }
+    return $overLoadedRule;
+}
+
+sub addPasswordPolicyDisplay {
+    my ( $self, $id, $options ) = @_;
+    $self->_ppRules->{$id} = {%$options};
+}
+
+sub _addPluginEntryPoint {
+    my ( $self, %entryPointDescription ) = @_;
+    push @{ $self->_pluginEntryPoints },
+      { _pkg => "[unknown]", %entryPointDescription };
 }
 
 1;
