@@ -18,11 +18,12 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
   PE_RENEWSESSION
   PE_UPGRADESESSION
+  PE_TOKENEXPIRED
 );
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.22.0';
 
 # PROPERTIES
 
@@ -95,12 +96,16 @@ sub _redirect {
         $self->logger->debug("Unauth request to $self->{path} issuer");
         $restore = 1;
         $self->logger->debug('Processing _redirect');
-        $ir = $req->pdata->{ $self->ipath } ||= $self->storeRequest($req);
+        if ( !$req->param("inProgress") ) {
+            $req->pdata->{ $self->ipath } = $self->storeRequest($req);
+        }
+        $self->p->setHiddenFormValue( $req, "inProgress", 1, '', 0 );
+        $ir = $req->pdata->{ $self->ipath };
         $req->pdata->{ $self->ipath . 'Path' } = \@path;
         $self->logger->debug(
             'Add ' . $self->ipath . ', ' . $self->ipath . 'Path in keepPdata' );
         push @{ $req->pdata->{keepPdata} }, $self->ipath, $self->ipath . 'Path';
-        $req->{urldc}           = $self->p->buildUrl( $self->path );
+        $req->{urldc}           = $self->p->buildUrl( $req->portal, $self->path );
         $req->pdata->{_url}     = encode_base64( $req->urldc, '' );
         $req->pdata->{issuerTs} = time;
     }
@@ -122,38 +127,58 @@ sub _redirect {
             @{ $self->p->endAuth },
             (
                 $restore
-                ? sub {
-
+                ? (
                     # Restore urldc if auth doesn't need to dial with browser
-                    $self->restoreRequest( $_[0], $ir );
-                    $self->cleanPdata( $_[0] );
-                    return $self->run( @_, @path );
-                  }
+                    [ ref($self) . "::resumeProcess", $ir, @path ],
+                  )
                 : ()
             )
         ]
     );
 }
 
+sub resumeProcess {
+    my ( $self, $req, $ir, @path ) = @_;
+
+    $self->p->clearHiddenFormValue($req, [ 'inProgress' ]);
+    $self->restoreRequest( $req, $ir );
+    $self->cleanPdata($req);
+    return $self->run( $req, @path );
+}
+
 # Case 3: authentified user, launch
 sub _forAuthUser {
     my ( $self, $req, @path ) = @_;
 
+    my $restore_failed;
     if (    $req->pdata->{issuerTs}
         and $req->pdata->{issuerTs} + $self->conf->{issuersTimeout} < time )
     {
+        $restore_failed = 1;
         $self->cleanAllPdata($req);
     }
+
     $self->logger->debug('Processing _forAuthUser');
     if ( my $r = $req->pdata->{ $self->ipath } ) {
         $self->logger->debug("Restoring request to $self->{path} issuer");
-        $self->restoreRequest( $req, $r );
-        @path = @{ $req->pdata->{ $self->ipath . 'Path' } }
-          if ( $req->pdata->{ $self->ipath . 'Path' } );
+        if ( $self->restoreRequest( $req, $r ) ) {
+            @path = @{ $req->pdata->{ $self->ipath . 'Path' } }
+              if ( $req->pdata->{ $self->ipath . 'Path' } );
 
-        # In case a confirm form is shown, we need it to POST on the
-        # current Path
-        $req->data->{confirmFormAction} = URI->new( $req->uri )->path;
+            # In case a confirm form is shown, we need it to POST on the
+            # current Path
+            $req->data->{confirmFormAction} = URI->new( $req->uri )->path;
+        }
+        else {
+            $restore_failed = 1;
+            $self->cleanAllPdata($req);
+        }
+    }
+
+    if ( $restore_failed and !@path ) {
+        $self->logger->error("Unable to restore issuer context");
+        $req->mustRedirect(1);
+        return $self->p->doPE($req, PE_TOKENEXPIRED);
     }
 
     # Clean pdata: keepPdata has been set, so pdata must be cleaned here
@@ -165,12 +190,8 @@ sub _forAuthUser {
     return $self->p->do(
         $req,
         [
-            'importHandlerData',
-            'controlUrl',
-            @{ $self->p->forAuthUser },
-            sub {
-                return $self->run( @_, @path );
-            },
+            'importHandlerData',        'controlUrl',
+            @{ $self->p->forAuthUser }, [ ref($self) . "::run", @path ],
         ]
     );
 }
@@ -208,7 +229,6 @@ sub cleanPdata {
 
 sub storeRequest {
     my ( $self, $req ) = @_;
-    $self->logger->debug('Store issuer request');
     my $info = {};
     $info->{content} = $req->content;
     foreach ( keys %{ $req->env } ) {
@@ -216,13 +236,19 @@ sub storeRequest {
         next if $_ eq "psgi.input";
         $info->{$_} = $req->env->{$_} unless ( ref $req->env->{$_} );
     }
-    return $self->_ott->createToken($info);
+    my $token = $self->_ott->createToken($info);
+    $self->logger->debug("Store issuer request as $token");
+    return $token;
 }
 
 sub restoreRequest {
     my ( $self, $req, $token ) = @_;
-    my $env = $self->_ott->getToken($token);
+    $self->logger->debug("Restore issuer request from $token");
+
+    my $result = 0;
+    my $env    = $self->_ott->getToken($token);
     if ($env) {
+        $result = 1;
         $self->logger->debug("Restoring request from $token");
         if ( my $c = delete $env->{content} ) {
             $env->{'psgix.input.buffered'} = 0;
@@ -237,16 +263,19 @@ sub restoreRequest {
     }
     $req->{uri} = uri_unescape( $req->env->{REQUEST_URI} );
     $req->{uri} =~ s|^//+|/|g;
-    return $req;
+    return $result;
 }
 
 sub reAuth {
     my ( $self, $req ) = @_;
+    my $cacheTag = $self->p->cacheTag;
     $req->data->{customScript} =
-qq'<script type="text/javascript" src="$self->{p}->{staticPrefix}/common/js/autoRenew.min.js"></script>'
+qq'<script type="text/javascript" src="$self->{p}->{staticPrefix}/common/js/autoRenew.min.js?v=$cacheTag"></script>'
       if ( $self->conf->{skipRenewConfirmation} );
+
+    my $portal_without_slash = $req->portal =~ s/\/+$//r;
     $req->data->{_url} =
-      encode_base64( $self->conf->{portal} . $req->path_info, '' );
+      encode_base64( $portal_without_slash . $req->path_info, '' );
     $req->pdata->{ $self->ipath } = $self->storeRequest($req);
     push @{ $req->pdata->{keepPdata} }, $self->ipath, $self->ipath . 'Path';
     $req->pdata->{issuerTs} = time;
@@ -255,11 +284,13 @@ qq'<script type="text/javascript" src="$self->{p}->{staticPrefix}/common/js/auto
 
 sub upgradeAuth {
     my ( $self, $req ) = @_;
+    my $cacheTag = $self->p->cacheTag;
     $req->data->{customScript} =
-qq'<script type="text/javascript" src="$self->{p}->{staticPrefix}/common/js/autoRenew.min.js"></script>'
+qq'<script type="text/javascript" src="$self->{p}->{staticPrefix}/common/js/autoRenew.min.js?v=$cacheTag"></script>'
       if ( $self->conf->{skipUpgradeConfirmation} );
+    my $portal_without_slash = $req->portal =~ s/\/+$//r;
     $req->data->{_url} =
-      encode_base64( $self->conf->{portal} . $req->path_info, '' );
+      encode_base64( $portal_without_slash . $req->path_info, '' );
     $req->pdata->{ $self->ipath } = $self->storeRequest($req);
     push @{ $req->pdata->{keepPdata} }, $self->ipath, $self->ipath . 'Path';
     $req->pdata->{issuerTs} = time;
