@@ -3,22 +3,23 @@ package Lemonldap::NG::Portal::Password::AD;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants qw(
-  PE_PASSWORD_OK
-  PE_LDAPERROR
-  PE_LDAPCONNECTFAILED
   PE_ERROR
+  PE_LDAPERROR
+  PE_PASSWORD_OK
+  PE_LDAPCONNECTFAILED
 );
 
-extends 'Lemonldap::NG::Portal::Lib::LDAP',
-  'Lemonldap::NG::Portal::Password::Base';
+extends qw(
+  Lemonldap::NG::Portal::Lib::LDAP
+  Lemonldap::NG::Portal::Password::Base
+);
 
-our $VERSION = '2.0.2';
+our $VERSION = '2.0.16';
 
 sub init {
     my ($self) = @_;
-    $self->ldap
-      and $self->filter
-      and $self->Lemonldap::NG::Portal::Password::Base::init;
+    return (  $self->Lemonldap::NG::Portal::Password::Base::init
+          and $self->Lemonldap::NG::Portal::Lib::LDAP::init );
 }
 
 # Confirmation is done by Lib::Net::LDAP::userModifyPassword
@@ -27,13 +28,26 @@ sub confirm {
 }
 
 sub modifyPassword {
-    my ( $self, $req, $pwd ) = @_;
-    my $dn = $req->userData->{_dn} || $req->sessionInfo->{_dn};
+    my ( $self, $req, $pwd, %args ) = @_;
+
+    # If the password change is done in a different backend,
+    # we need to reload the correct DN
+    $self->getUser( $req, useMail => $args{useMail} )
+      if $self->conf->{ldapGetUserBeforePasswordChange};
+
+    my $dn = $req->data->{dn} || $req->sessionInfo->{_dn};
     unless ($dn) {
-        $self->logger->error('"dn" is not set, aborting password modification');
+        $self->logger->error('"dn" is not set, abort password modification');
         return PE_ERROR;
     }
 
+    my $requireOldPassword = (
+          $req->userData
+        ? $self->requireOldPwdRule->( $req, $req->userData )
+        : $self->requireOldPwdRule->( $req, $req->sessionInfo )
+    );
+    $requireOldPassword = 0 if $args{passwordReset};
+
     # Ensure connection is valid
     $self->bind;
     return PE_LDAPCONNECTFAILED unless $self->ldap;
@@ -41,11 +55,8 @@ sub modifyPassword {
     # Call the modify password method
     my $code =
       $self->ldap->userModifyPassword( $dn, $pwd, $req->data->{oldpassword},
-        1 );
-
-    unless ( $code == PE_PASSWORD_OK ) {
-        return $code;
-    }
+        1, $requireOldPassword );
+    return $code unless ( $code == PE_PASSWORD_OK );
 
     # If force reset, set reset flag
     if ( $req->data->{forceReset} ) {
