@@ -7,7 +7,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_OK
 );
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.8';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
@@ -17,7 +17,11 @@ use constant beforeAuth => 'check';
 
 # INITIALIZATION
 
-has rules => ( is => 'rw', default => sub { [] } );
+has rules => (
+    is      => 'rw',
+    isa     => 'ArrayRef',
+    default => sub { [] }
+);
 
 sub init {
     my ($self) = @_;
@@ -25,15 +29,21 @@ sub init {
         my $safe = Safe->new;
         foreach my $id ( sort keys %$rules ) {
             my $sub =
-              $safe->reval( 'sub{my($env)=@_;return (' . $rules->{$id} . ')}' );
+              $safe->reval(
+                'sub{my($env)=@_;local *_;return (' . $rules->{$id} . ')}' );
             if ($@) {
-                $self->error( 'Bad Autologin rule "' . $rules->{$id} . ': $@' );
-                return 0;
+                $self->logger->error(
+                    'Bad Autologin rule "' . $rules->{$id} . ": $@" );
+                $self->logger->debug(
+                    "Skipping Autologin rule for user \"$id\"");
+                next;
             }
+            $self->logger->debug("Autologin rule for user \"$id\" appended");
             $id =~ s/^\s*([\w\-\@]+)\s*/$1/;
             push @{ $self->rules }, [ $sub, $id ];
         }
     }
+
     return 1;
 }
 
