@@ -8,7 +8,7 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SESSIONNOTGRANTED
 );
 
-our $VERSION = '2.0.10';
+our $VERSION = '2.19.0';
 
 extends 'Lemonldap::NG::Portal::Main::Plugin';
 
@@ -36,7 +36,7 @@ sub run {
     sub sortByComment {
         my $A = ( $a =~ /^.*?##(.*)$/ )[0];
         my $B = ( $b =~ /^.*?##(.*)$/ )[0];
-        return !$A ? 1 : !$B ? -1 : $A cmp $B;
+        return !( $A or $B ) ? $a cmp $b : !$A ? 1 : !$B ? -1 : $A cmp $B;
     }
 
     # Avoid display notification if AuthResult is not null
@@ -76,7 +76,8 @@ sub run {
                       . $req->{sessionInfo}->{ $self->conf->{whatToTrace} }
                       . '" was not granted to open session (rule ->'
                       . "$rule)" );
-                $req->urldc( $self->conf->{portal} );
+                $req->urldc(
+                    $self->p->buildUrl( $req->portal, { cancel => 1 } ) );
                 return $req->authResult(PE_SESSIONNOTGRANTED);
             }
             else {
@@ -85,7 +86,7 @@ sub run {
                       . '" was not granted to open session (rule -> '
                       . $self->conf->{grantSessionRules}->{$_}
                       . ")" );
-                $req->urldc( $self->conf->{portal} );
+                $req->urldc( $req->portal );
                 return $req->authResult(PE_SESSIONNOTGRANTED);
             }
         }
