@@ -4,29 +4,23 @@ use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Main::Constants 'PE_OK';
 
-extends 'Lemonldap::NG::Common::Module';
+extends 'Lemonldap::NG::Portal::Main::UserDB';
 
-our $VERSION = '2.0.12';
+our $VERSION = '2.19.0';
 
 # INITIALIZATION
 
 sub init {
     my ($self) = @_;
     if ( $self->conf->{proxyUseSoap} ) {
-        extends 'Lemonldap::NG::Common::Module',
+        extends 'Lemonldap::NG::Portal::Main::UserDB',
           'Lemonldap::NG::Portal::Lib::SOAPProxy';
     }
     else {
-        extends 'Lemonldap::NG::Common::Module',
+        extends 'Lemonldap::NG::Portal::Main::UserDB',
           'Lemonldap::NG::Portal::Lib::RESTProxy';
     }
     return $self->SUPER::init();
 }
 
-# RUNNING METHODS
-
-sub setGroups {
-    return PE_OK;
-}
-
 1;
