@@ -228,11 +228,10 @@
         return results;
       } catch (error1) {
         e = error1;
-        console.log('Parsing error', e);
-        return console.log('JSON', $(this).text());
+        console.error('Parsing error', e);
+        console.debug('JSON', $(this).text());
       }
     });
-    console.log(values);
     return values;
   };
 
@@ -449,7 +448,7 @@
     }
     // If there are no auto-focused fields, focus on first visible input
     if ($("input[autofocus]").length === 0) {
-      $("input[type!=hidden]:first").focus();
+      $("input[type=text], input[type=password]").first().focus();
     }
     // Open links in new windows if required
     if (datas['newwindow']) {
@@ -459,18 +458,18 @@
     if ($("p.removeOther").length) {
       action = $("#form").attr("action");
       method = $("#form").attr("method");
-      console.log('method=', method);
+      console.debug('method=', method);
       hiddenParams = "";
       if ($("#form input[type=hidden]")) {
-        console.log('Parse hidden values');
+        console.debug('Parse hidden values');
         $("#form input[type=hidden]").each(function (index) {
-          console.log(' ->', $(this).attr("name"), $(this).val());
+          console.debug(' ->', $(this).attr("name"), $(this).val());
           return hiddenParams += "&" + $(this).attr("name") + "=" + $(this).val();
         });
       }
       back_url = "";
       if (action) {
-        console.log('action=', action);
+        console.debug('action=', action);
         if (action.indexOf("?") !== -1) {
           action.substring(0, action.indexOf("?")) + "?";
         } else {
@@ -491,37 +490,37 @@
     if (window.location.search) {
       queryLang = getQueryParam('llnglanguage');
       if (queryLang) {
-        console.log('Get lang from parameter');
+        console.debug('Get lang from parameter');
       }
       setCookieLang = getQueryParam('setCookieLang');
       if (setCookieLang === 1) {
-        console.log('Set lang cookie');
+        console.debug('Set lang cookie');
       }
     }
     if (!lang) {
       lang = window.datas['language'];
       if (lang && !queryLang) {
-        console.log('Get lang from server');
+        console.debug('Get lang from server');
       }
     } else if (indexOf.call(window.availableLanguages, lang) < 0) {
       lang = window.datas['language'];
       if (!queryLang) {
-        console.log('Lang not available -> Get lang from server');
+        console.debug('Lang not available -> Get lang from server');
       }
     }
     if (queryLang) {
       if (indexOf.call(window.availableLanguages, queryLang) < 0) {
-        console.log('Lang not available -> Get lang from server');
+        console.debug('Lang not available -> Get lang from server');
         queryLang = window.language;
       }
-      console.log('Selected lang ->', queryLang);
+      console.debug('Selected lang ->', queryLang);
       if (setCookieLang) {
-        console.log('Set cookie lang ->', queryLang);
+        console.debug('Set cookie lang ->', queryLang);
         setCookie('llnglanguage', queryLang, 3650);
       }
       translatePage(queryLang);
     } else {
-      console.log('Selected lang ->', lang);
+      console.debug('Selected lang ->', lang);
       translatePage(lang);
     }
     // Build language icons
@@ -621,7 +620,7 @@
         $(".toggle-password").on('mousedown touchstart', function () {
           field = $(this).attr('id');
           field = field.replace(/^toggle_/, '');
-          console.log('Display', field);
+          console.debug('Display', field);
           $(this).toggleClass("fa-eye fa-eye-slash");
           return $("input[name=".concat(field, "]")).attr('class', 'form-control');
         });
@@ -635,7 +634,7 @@
         $(".toggle-password").on('mousedown touchstart', function () {
           field = $(this).attr('id');
           field = field.replace(/^toggle_/, '');
-          console.log('Display', field);
+          console.debug('Display', field);
           $(this).toggleClass("fa-eye fa-eye-slash");
           return $("input[name=".concat(field, "]")).attr("type", "text");
         });
@@ -649,7 +648,7 @@
     $('#reset').change(function () {
       var checked, ref1, ref2, ref3, ref4, ref5;
       checked = $(this).prop('checked');
-      console.log('Reset is checked', checked);
+      console.debug('Reset is checked', checked);
       if (checked === true) {
         $('#ppolicy').hide();
         $('#newpasswords').hide();
@@ -700,12 +699,12 @@
     //$('#formpass').on 'submit', changePwd
     $('.clear-finduser-field').on('click', function () {
       return $(this).parent().find(':input').each(function () {
-        console.log('Clear search field ->', $(this).attr('name'));
+        console.debug('Clear search field ->', $(this).attr('name'));
         return $(this).val('');
       });
     });
     $('#closefinduserform').on('click', function () {
-      console.log('Clear modal');
+      console.debug('Clear modal');
       return $('#finduserForm').trigger('reset');
     });
     $('#finduserbutton').on('click', function (event) {
@@ -713,7 +712,7 @@
       event.preventDefault();
       document.body.style.cursor = 'progress';
       str = $("#finduserForm").serialize();
-      console.log('Send findUser request with parameters', str);
+      console.debug('Send findUser request with parameters', str);
       return $.ajax({
         type: "POST",
         url: "".concat(scriptname, "finduser"),
@@ -724,7 +723,7 @@
           var user;
           document.body.style.cursor = 'default';
           user = data.user;
-          console.log('Suggested spoofId=', user);
+          console.debug('Suggested spoofId=', user);
           $("input[name=spoofId]").each(function () {
             return $(this).val(user);
           });
@@ -740,19 +739,19 @@
           var res;
           document.body.style.cursor = 'default';
           if (err) {
-            console.log('Error', err);
+            console.error('Error', err);
           }
           if (j) {
             res = JSON.parse(j.responseText);
           }
           if (res && res.error) {
-            return console.log('Returned error', res);
+            console.error('Returned error', res);
           }
         }
       });
     });
     $('#btn-back-to-top').on('click', function () {
-      console.log('Back to top');
+      console.debug('Back to top');
       document.body.scrollTop = 0;
       return document.documentElement.scrollTop = 0;
     });
@@ -763,16 +762,24 @@
         return $('#btn-back-to-top').css("display", "none");
       }
     });
-    $('.btn-single-submit').on('click', function (event) {
+    $('form[data-property=single-submit]').on('submit', function (event) {
       if ($(this).data('data-submitted') === true) {
         event.preventDefault();
-        return $(this).prop('disabled', true);
       } else {
+        $(this).find(':submit').prop('disabled', true);
         return $(this).data('data-submitted', true);
       }
     });
     $(".category[name=\"".concat(datas['floatingCategory'], "\"]")).appendTo('#floating-menu').find("i").remove();
     $(".category[name=\"".concat(datas['floatingCategory'], "\"]")).draggable();
+    var parent = document.getElementById('floating-menu');
+    if (parent) {
+      var divs = parent.querySelectorAll('.col-md-4');
+      divs.forEach(function (div, index) {
+        div.classList.remove("col-md-4");
+        div.classList.add("col-md-12");
+      });
+    }
     $(document).trigger("portalLoaded");
     return true;
   });
